//
//  TekpubTodoViewController.m
//  TekpubTodo
//
//  Created by Andrew Gaskell on 3/12/11.
//  Copyright 2011 Zubago Inc. All rights reserved.
//

#import "TekpubTodoViewController.h"
#import "TodoEditorController.h"
#import "Todo.h"
#import "JSON.h"

@implementation TekpubTodoViewController

- (void)addTodo:(NSString *)text {
    Todo *todo = [[Todo alloc] initWithText:text];
    [items addObject:todo];
    [todo release];
}

- (void)parser:(SBJsonStreamParser *)parser foundArray:(NSArray *)array {
    NSLog(@"ArrayTweet: '%@'", array);	
    
    [items removeAllObjects];
    
    for (NSDictionary *user in array) {
        NSString *str = [NSString stringWithFormat:@"%@", [user objectForKey:@"Email"]];
        [self addTodo:str];
    }
    
    [_tableView reloadData];
}

- (void)parser:(SBJsonStreamParser *)parser foundObject:(NSDictionary *)dict {
	NSString *text = [dict objectForKey:@"Fullname"];
	NSLog(@"Tweet: '%@'", text);
}


- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

-(void) presentTodoEditor:(NSString *)text {
    TodoEditorController *editor = [[TodoEditorController alloc] initWithNibName:@"TodoEditorController" bundle:nil];
    editor.defaultText = text;
    editor.delegate = self;
    
    [self presentModalViewController:editor animated:YES];
    [editor release];
}

-(void) addButtonPushed {
    lastEditedTodo = nil;
    [self presentTodoEditor:nil];
}

-(IBAction)edit {
    editing = !editing;
    [_tableView setEditing:editing animated:YES];
    
  
}


-(void)todoEditor:(TodoEditorController *)editor didFinishWithResult:(BOOL)result {
    if(result) {
        NSString *text = editor.text;
        
        if (lastEditedTodo == nil) {
            Todo *todo = [[Todo alloc] initWithText:text];
            [items addObject:todo];
            [todo release];
        } else {
            lastEditedTodo.text = text;
        }
        
        
        [_tableView reloadData];
    }
    
    [self dismissModalViewControllerAnimated:YES];
}

#pragma mark - View lifecycle


- (id)initWithCoder:(NSCoder *)aDecoder {
    if(self = [super initWithCoder:aDecoder]) {
        items = [[NSMutableArray alloc] init];
        
        [self addTodo:@"Take out the trash"];
        [self addTodo:@"Do the dishes"];        
        [self addTodo:@"Record a screencast"];
    }
    
    return self;
}

// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad
{
    [super viewDidLoad];
    
	adapter = [SBJsonStreamParserAdapter new];
	
	// Set ourselves as the delegate, so we receive the messages
	// from the adapter.
	adapter.delegate = self;
	
	// Create a new stream parser..
	parser = [SBJsonStreamParser new];
	
	// .. and set our adapter as its delegate.
	parser.delegate = adapter;
	
	// Normally it's an error if JSON is followed by anything but
	// whitespace. Setting this means that the parser will be
	// expecting the stream to contain multiple whitespace-separated
	// JSON documents.
	parser.multi = YES;
    
    
    NSString *urlString = @"http://cirrus.cumminspg.com/users.json";
    
    NSURL *url = [NSURL URLWithString:urlString];
    
    NSURLRequest *request = [[NSURLRequest alloc] initWithURL:url];
    
    
    NSURLConnection *connection = [[NSURLConnection alloc] initWithRequest:request delegate:self];
    
    
    [connection release];
    [request release];
}

-(void) connection:(NSURLConnection *)connection didReceiveAuthenticationChallenge:(NSURLAuthenticationChallenge *)challenge {
    NSURLCredential *newCredential;
    newCredential = [NSURLCredential credentialWithUser:@"andy" password:@"andy" persistence:NSURLCredentialPersistenceForSession];
    [[challenge sender] useCredential:newCredential forAuthenticationChallenge:challenge];
}

-(void) connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
    
	NSLog(@"Connection didReceiveData of length: %u", data.length);
	
	// Parse the new chunk of data. The parser will append it to
	// its internal buffer, then parse from where it left off in
	// the last chunk.
	SBJsonStreamParserStatus status = [parser parse:data];
	
	if (status == SBJsonStreamParserError) {
		NSLog(@"Parser error: %@", parser.error);
		
	} else if (status == SBJsonStreamParserWaitingForData) {
		NSLog(@"Parser waiting for more data");
	}
  
}


- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (void)dealloc {
    [items dealloc];
    
    [super dealloc];
}

#pragma mark - TableView datasource methods

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return [items count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    NSString *cellIdentifier = @"cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellIdentifier];
    if(cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault 
                                       reuseIdentifier:cellIdentifier] autorelease];        
    }
    
    
    Todo *todo = [items objectAtIndex:indexPath.row];
    cell.textLabel.text = todo.text;
    
    if(todo.completed)
        cell.accessoryType = UITableViewCellAccessoryCheckmark;
    else
        cell.accessoryType = UITableViewCellAccessoryNone;
    
    return cell;
}

-(void) tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        [items removeObjectAtIndex:indexPath.row];
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationRight];
        
         
    }
}


#pragma mark - TableView delegate methods

- (void)tableView:(UITableView *)tableView willBeginEditingRowAtIndexPath:(NSIndexPath *)indexPath {
    Todo *todo = [items objectAtIndex:indexPath.row];
    [self presentTodoEditor:todo.text];
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {    
    Todo *todo = [items objectAtIndex:indexPath.row];
    
    if (editing) {
        lastEditedTodo = todo;
        [self presentTodoEditor:todo.text];        
    } else {
        todo.completed = !todo.completed;
        [tableView reloadData];
    }
    
    [tableView deselectRowAtIndexPath:indexPath animated:YES];

}

#pragma mark - cleanup


@end
